@@ -45,7 +45,7 @@ class RConnector(object):
         return self.__closed
 
     def connect(self):
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.sock = socket.socket()
         try:
             self.sock.connect((self.host, self.port))
         except socket.error:
@@ -79,13 +79,13 @@ class RConnector(object):
         self._reval(aString)
         if DEBUG:
             # Read entire data into memory en block, it's easier to debug
-            src = self.receive()
+            src = self._receive()
             print 'Raw response:', repr(src)
         else:
             src = self.sock.makefile()
         try:
             return rparse(src)
-        except REvalError, msg:
+        except REvalError:
             # R has reported an evaulation error, so let's obtain a descriptive explanation
             # about why the error has occurred. R allows to retrieve the error message
             # of the last exception via a built-in function called 'geterrmessage()'.
@@ -159,7 +159,7 @@ class RConnector(object):
     def assign(self, aDict):
         '@brief Assign all items of the dictionary to the default R namespace'
         for k, v in aDict.items():
-            setRexp(k, v)
+            self.setRexp(k, v)
 
     @checkIfClosed
     def isFunction(self, name):
@@ -214,33 +214,33 @@ class RNameSpaceReference(object):
 class RBaseProxy(object):
     'Proxy for a reference to a variable or function in R. Do not use this directly, only its subclasses'
     def __init__(self, name, rconn):
-        self.name = name
-        self.rconn = rconn
+        self._name = name
+        self._rconn = rconn
 
     
 
 class RVarProxy(RBaseProxy):
     'Proxy for a reference to a variable in R'
     def __repr__(self):
-        return '<RVarProxy to variable "%s">' % self.name
+        return '<RVarProxy to variable "%s">' % self._name
 
     def value(self):
-        return self.rconn.getRexp(self.name)
+        return self._rconn.getRexp(self._name)
         
 
 
 class RFuncProxy(RBaseProxy):
     'Proxy for function calls to Rserve'
     def __repr__(self):
-        return '<RFuncProxy to function "%s">' % self.name
+        return '<RFuncProxy to function "%s">' % self._name
         
     def __call__(self, *args, **kw):
-        return self.rconn.callFunc(self.name, *args, **kw)
+        return self._rconn.callFunc(self._name, *args, **kw)
 
     @property
     def __doc__(self):
         try:
-            d = self.rconn.eval('readLines(as.character(help(%s)))' % self.name)
+            d = self._rconn.eval('readLines(as.character(help(%s)))' % self._name)
         except REvalError:
             # probably no help available, unfortunately there is no specific code for this...
             return None
@@ -251,7 +251,15 @@ class RFuncProxy(RBaseProxy):
     def help(self):
         print self.__doc__
 
-
+    def __getattr__(self, name):
+        """Allow for nested name space calls, e.g. 't.test' """
+        concatName = "%s.%s" % (self._name, name)
+        try:
+            isFunction = self._rconn.isFunction(concatName)
+        except:
+            # an error is only raised if neither such a function or variable exists at all!
+            raise NameError('no such variable or function "%s" defined in Rserve' % concatName)
+        return RFuncProxy(concatName, self._rconn)
 
 
 if __name__ == '__main__':
